@@ -31,15 +31,17 @@
 
 #include <functional>
 
-namespace Utils { class FileName; }
+namespace Utils { class FilePath; }
 
 namespace ProjectExplorer {
+class BuildSystem;
 class FileNode;
 class FolderNode;
 class Node;
 class Project;
 class ProjectNode;
 class SessionNode;
+class Target;
 
 namespace Internal { class ProjectTreeWidget; }
 
@@ -53,8 +55,10 @@ public:
     static ProjectTree *instance();
 
     static Project *currentProject();
-    static Node *findCurrentNode();
-    static Utils::FileName currentFilePath();
+    static Target *currentTarget();
+    static BuildSystem *currentBuildSystem();
+    static Node *currentNode();
+    static Utils::FilePath currentFilePath();
 
     // Integration with ProjectTreeWidget
     static void registerWidget(Internal::ProjectTreeWidget *widget);
@@ -76,17 +80,24 @@ public:
     static void forEachNode(const std::function<void(Node *)> &task);
 
     static Project *projectForNode(const Node *node);
-    static Node *nodeForFile(const Utils::FileName &fileName);
+    static Node *nodeForFile(const Utils::FilePath &fileName);
+
+    static const QList<Node *> siblingsWithSameBaseName(const Node *fileNode);
+
+    void expandCurrentNodeRecursively();
 
     void collapseAll();
     void expandAll();
 
+    void changeProjectRootDirectory();
+
     // for nodes to emit signals, do not call unless you are a node
     static void emitSubtreeChanged(FolderNode *node);
 
 signals:
     void currentProjectChanged(ProjectExplorer::Project *project);
     void currentNodeChanged();
+    void nodeActionsChanged();
 
     // Emitted whenever the model needs to send a update signal.
     void subtreeChanged(ProjectExplorer::FolderNode *node);
@@ -111,6 +122,7 @@ private:
 
     void updateExternalFileWarning();
     static bool hasFocus(Internal::ProjectTreeWidget *widget);
+    Internal::ProjectTreeWidget *currentWidget() const;
     void hideContextMenu();
 
 private:
