@@ -41,10 +41,14 @@
 #include <qmldesignerplugin.h>
 #include <viewmanager.h>
 
+#include <listmodeleditor/listmodeleditordialog.h>
+#include <listmodeleditor/listmodeleditormodel.h>
+
 #include <QHBoxLayout>
 #include <QGraphicsLinearLayout>
 
 #include <coreplugin/actionmanager/actionmanager.h>
+#include <coreplugin/icore.h>
 #include <utils/algorithm.h>
 #include <utils/qtcassert.h>
 #include <utils/utilsicons.h>
@@ -112,16 +116,17 @@ void DesignerActionManager::polishActions() const
                                                         [](ActionInterface *action) { return action->type() != ActionInterface::ContextMenu; });
 
     Core::Context qmlDesignerFormEditorContext(Constants::C_QMLFORMEDITOR);
+    Core::Context qmlDesignerEditor3DContext(Constants::C_QMLEDITOR3D);
     Core::Context qmlDesignerNavigatorContext(Constants::C_QMLNAVIGATOR);
 
     Core::Context qmlDesignerUIContext;
     qmlDesignerUIContext.add(qmlDesignerFormEditorContext);
+    qmlDesignerUIContext.add(qmlDesignerEditor3DContext);
     qmlDesignerUIContext.add(qmlDesignerNavigatorContext);
 
     for (auto *action : actions) {
         if (!action->menuId().isEmpty()) {
-            const QString id =
-                    QString("QmlDesigner.%1").arg(QString::fromLatin1(action->menuId()));
+            const QString id = QString("QmlDesigner.%1").arg(QString::fromLatin1(action->menuId()));
 
             Core::Command *cmd = Core::ActionManager::registerAction(action->action(), id.toLatin1().constData(), qmlDesignerUIContext);
 
@@ -334,6 +339,145 @@ public:
     }
 };
 
+class EditListModelAction final : public ModelNodeContextMenuAction
+{
+public:
+    EditListModelAction()
+        : ModelNodeContextMenuAction("EditListModel",
+                                     ComponentCoreConstants::editListModelDisplayName,
+                                     {},
+                                     ComponentCoreConstants::rootCategory,
+                                     QKeySequence("Alt+e"),
+                                     1001,
+                                     &openDialog,
+                                     &isListViewInBaseState,
+                                     &isListViewInBaseState)
+    {}
+
+    static bool isListViewInBaseState(const SelectionContext &selectionState)
+    {
+        return selectionState.isInBaseState() && selectionState.singleNodeIsSelected()
+               && selectionState.currentSingleSelectedNode().metaInfo().isSubclassOf(
+                   "QtQuick.ListView");
+    }
+
+    bool isEnabled(const SelectionContext &) const override { return true; }
+
+    static void openDialog(const SelectionContext &selectionState)
+    {
+        ModelNode targetNode = selectionState.targetNode();
+        if (!targetNode.isValid())
+            targetNode = selectionState.currentSingleSelectedNode();
+        if (!targetNode.isValid())
+            return;
+
+        AbstractView *view = targetNode.view();
+        NodeMetaInfo modelMetaInfo = view->model()->metaInfo("ListModel");
+        NodeMetaInfo elementMetaInfo = view->model()->metaInfo("ListElement");
+
+        ListModelEditorModel model{[&] {
+                                       return view->createModelNode(modelMetaInfo.typeName(),
+                                                                    modelMetaInfo.majorVersion(),
+                                                                    modelMetaInfo.minorVersion());
+                                   },
+                                   [&] {
+                                       return view->createModelNode(elementMetaInfo.typeName(),
+                                                                    elementMetaInfo.majorVersion(),
+                                                                    elementMetaInfo.minorVersion());
+                                   }};
+
+        model.setListView(targetNode);
+
+        ListModelEditorDialog dialog{Core::ICore::mainWindow()};
+        dialog.setModel(&model);
+
+        dialog.exec();
+    }
+};
+
+bool flowOptionVisible(const SelectionContext &context)
+{
+    return QmlFlowViewNode::isValidQmlFlowViewNode(context.rootNode());
+}
+
+bool isFlowItem(const SelectionContext &context)
+{
+    return context.singleNodeIsSelected()
+           && QmlFlowItemNode::isValidQmlFlowItemNode(context.currentSingleSelectedNode());
+}
+
+bool isFlowTarget(const SelectionContext &context)
+{
+    return context.singleNodeIsSelected()
+           && QmlFlowTargetNode::isFlowEditorTarget(context.currentSingleSelectedNode());
+}
+
+bool isFlowTransitionItem(const SelectionContext &context)
+{
+    return context.singleNodeIsSelected()
+           && QmlFlowItemNode::isFlowTransition(context.currentSingleSelectedNode());
+}
+
+bool isFlowTransitionItemWithEffect(const SelectionContext &context)
+{
+    if (!isFlowTransitionItem(context))
+        return false;
+
+    ModelNode node = context.currentSingleSelectedNode();
+
+    return node.hasNodeProperty("effect");
+}
+
+bool isFlowActionItemItem(const SelectionContext &context)
+{
+    const ModelNode selectedNode = context.currentSingleSelectedNode();
+
+    return context.singleNodeIsSelected()
+            && (QmlFlowActionAreaNode::isValidQmlFlowActionAreaNode(selectedNode)
+                || QmlVisualNode::isFlowDecision(selectedNode)
+                || QmlVisualNode::isFlowWildcard(selectedNode));
+}
+
+bool isFlowTargetOrTransition(const SelectionContext &context)
+{
+    return isFlowTarget(context) || isFlowTransitionItem(context);
+}
+
+class FlowActionConnectAction : public ActionGroup
+{
+public:
+    FlowActionConnectAction(const QString &displayName, const QByteArray &menuId, int priority) :
+        ActionGroup(displayName, menuId, priority,
+                    &isFlowActionItemItem, &flowOptionVisible)
+
+    {}
+
+    void updateContext() override
+    {
+        menu()->clear();
+        if (selectionContext().isValid()) {
+            action()->setEnabled(isEnabled(selectionContext()));
+            action()->setVisible(isVisible(selectionContext()));
+        } else {
+            return;
+        }
+        if (action()->isEnabled()) {
+            for (const QmlFlowItemNode &node : QmlFlowViewNode(selectionContext().rootNode()).flowItems()) {
+                if (node != selectionContext().currentSingleSelectedNode().parentProperty().parentModelNode()) {
+                    QString what = QString(QT_TRANSLATE_NOOP("QmlDesignerContextMenu", "Connect: %1")).arg(captionForModelNode(node));
+                    ActionTemplate *connectionAction = new ActionTemplate(what, &ModelNodeOperations::addTransition);
+
+                    SelectionContext nodeSelectionContext = selectionContext();
+                    nodeSelectionContext.setTargetNode(node);
+                    connectionAction->setSelectionContext(nodeSelectionContext);
+
+                    menu()->addAction(connectionAction);
+                }
+            }
+        }
+    }
+};
+
 const char xProperty[] = "x";
 const char yProperty[] = "y";
 const char zProperty[] = "z";
@@ -514,6 +658,27 @@ bool isStackedContainerAndIndexCanBeIncreased(const SelectionContext &context)
     return value < maxValue;
 }
 
+bool isGroup(const SelectionContext &context)
+{
+    if (!inBaseState(context))
+        return false;
+
+    if (!singleSelection(context))
+        return false;
+
+    ModelNode currentSelectedNode = context.currentSingleSelectedNode();
+
+    if (!currentSelectedNode.isValid())
+        return false;
+
+    NodeMetaInfo metaInfo = currentSelectedNode.metaInfo();
+
+    if (!metaInfo.isValid())
+        return false;
+
+    return metaInfo.isSubclassOf("QtQuick.Studio.Components.GroupItem");
+}
+
 bool isLayout(const SelectionContext &context)
 {
     if (!inBaseState(context))
@@ -574,6 +739,12 @@ bool positionOptionVisible(const SelectionContext &context)
             || isPositioner(context);
 }
 
+bool studioComponentsAvailable(const SelectionContext &context)
+{
+    const Import import = Import::createLibraryImport("QtQuick.Studio.Components", "1.0");
+    return context.view()->model()->isImportPossible(import, true, true);
+}
+
 bool singleSelectedAndUiFile(const SelectionContext &context)
 {
     if (!singleSelection(context))
@@ -785,6 +956,78 @@ void DesignerActionManager::createDefaultDesignerActions()
                           priorityLayoutCategory,
                           &layoutOptionVisible));
 
+    addDesignerAction(new ActionGroup(groupCategoryDisplayName,
+                                      groupCategory,
+                                      priorityGroupCategory,
+                                      &studioComponentsAvailable));
+
+    addDesignerAction(new ActionGroup(
+        flowCategoryDisplayName,
+        flowCategory,
+        priorityFlowCategory,
+        &isFlowTargetOrTransition,
+        &flowOptionVisible));
+
+
+    auto effectMenu = new ActionGroup(
+                flowEffectCategoryDisplayName,
+                flowEffectCategory,
+                priorityFlowCategory,
+                &isFlowTransitionItem,
+                &flowOptionVisible);
+
+    effectMenu->setCategory(flowCategory);
+    addDesignerAction(effectMenu);
+
+    addDesignerAction(new ModelNodeFormEditorAction(
+        createFlowActionAreaCommandId,
+        createFlowActionAreaDisplayName,
+        addIcon.icon(),
+        addFlowActionToolTip,
+        flowCategory,
+        {},
+        priorityFirst,
+        &createFlowActionArea,
+        &isFlowItem,
+        &flowOptionVisible));
+
+    addDesignerAction(new ModelNodeContextMenuAction(
+                          setFlowStartCommandId,
+                          setFlowStartDisplayName,
+                          {},
+                          flowCategory,
+                          priorityFirst,
+                          {},
+                          &setFlowStartItem,
+                          &isFlowItem,
+                          &flowOptionVisible));
+
+    addDesignerAction(new FlowActionConnectAction(
+        flowConnectionCategoryDisplayName,
+        flowConnectionCategory,
+        priorityFlowCategory));
+
+
+    const QList<TypeName> transitionTypes = {"FlowFadeEffect",
+                                   "FlowPushEffect",
+                                   "FlowMoveEffect",
+                                   "None"};
+
+    for (const TypeName &typeName : transitionTypes)
+        addTransitionEffectAction(typeName);
+
+    addCustomTransitionEffectAction();
+
+    addDesignerAction(new ModelNodeContextMenuAction(
+        selectFlowEffectCommandId,
+        selectEffectDisplayName,
+        {},
+        flowCategory,
+        {},
+        priorityFlowCategory,
+        &selectFlowEffect,
+        &isFlowTransitionItemWithEffect));
+
     addDesignerAction(new ActionGroup(
                           stackedContainerCategoryDisplayName,
                           stackedContainerCategory,
@@ -859,17 +1102,34 @@ void DesignerActionManager::createDefaultDesignerActions()
                           &isLayout,
                           &isLayout));
 
-    addDesignerAction(new ModelNodeFormEditorAction(
-                          addItemToStackedContainerCommandId,
-                          addItemToStackedContainerDisplayName,
-                          addIcon.icon(),
-                          addItemToStackedContainerToolTip,
-                          stackedContainerCategory,
-                          QKeySequence("Ctrl+Shift+a"),
-                          110,
-                          &addItemToStackedContainer,
-                          &isStackedContainer,
-                          &isStackedContainer));
+    addDesignerAction(new ModelNodeContextMenuAction(addToGroupItemCommandId,
+                                                     addToGroupItemDisplayName,
+                                                     {},
+                                                     groupCategory,
+                                                     QKeySequence("Ctrl+Shift+g"),
+                                                     110,
+                                                     &addToGroupItem,
+                                                     &selectionCanBeLayouted));
+
+    addDesignerAction(new ModelNodeContextMenuAction(removeGroupItemCommandId,
+                                                     removeGroupItemDisplayName,
+                                                     {},
+                                                     groupCategory,
+                                                     QKeySequence(),
+                                                     110,
+                                                     &removeGroup,
+                                                     &isGroup));
+
+    addDesignerAction(new ModelNodeFormEditorAction(addItemToStackedContainerCommandId,
+                                                    addItemToStackedContainerDisplayName,
+                                                    addIcon.icon(),
+                                                    addItemToStackedContainerToolTip,
+                                                    stackedContainerCategory,
+                                                    QKeySequence("Ctrl+Shift+a"),
+                                                    110,
+                                                    &addItemToStackedContainer,
+                                                    &isStackedContainer,
+                                                    &isStackedContainer));
 
     addDesignerAction(new ModelNodeContextMenuAction(
                           addTabBarToStackedContainerCommandId,
@@ -1004,12 +1264,23 @@ void DesignerActionManager::createDefaultDesignerActions()
                           &singleSelection,
                           &singleSelection));
 
+    addDesignerAction(new ModelNodeContextMenuAction(mergeTemplateCommandId,
+                                                     mergeTemplateDisplayName,
+                                                     {},
+                                                     rootCategory,
+                                                     {},
+                                                     50,
+                                                     &mergeWithTemplate,
+                                                     &SelectionContextFunctors::always));
+
     addDesignerAction(new ActionGroup(
                           "",
                           genericToolBarCategory,
                           priorityGenericToolBar));
 
     addDesignerAction(new ChangeStyleAction());
+
+    addDesignerAction(new EditListModelAction);
 }
 
 void DesignerActionManager::createDefaultAddResourceHandler()
@@ -1026,6 +1297,13 @@ void DesignerActionManager::createDefaultAddResourceHandler()
     registerAddResourceHandler(AddResourceHandler(ComponentCoreConstants::addImagesDisplayString,
                                                   "*.svg",
                                                   ModelNodeOperations::addImageToProject));
+
+    registerAddResourceHandler(AddResourceHandler(ComponentCoreConstants::addFontsDisplayString,
+                                                  "*.ttf",
+                                                  ModelNodeOperations::addFontToProject));
+    registerAddResourceHandler(AddResourceHandler(ComponentCoreConstants::addFontsDisplayString,
+                                                  "*.otf",
+                                                  ModelNodeOperations::addFontToProject));
 }
 
 void DesignerActionManager::addDesignerAction(ActionInterface *newAction)
@@ -1053,9 +1331,40 @@ DesignerActionManager::DesignerActionManager(DesignerActionManagerView *designer
 
 DesignerActionManager::~DesignerActionManager() = default;
 
+void DesignerActionManager::addTransitionEffectAction(const TypeName &typeName)
+{
+    addDesignerAction(new ModelNodeContextMenuAction(
+        QByteArray(ComponentCoreConstants::flowAssignEffectCommandId) + typeName,
+        QLatin1String(ComponentCoreConstants::flowAssignEffectDisplayName) + typeName,
+        {},
+        ComponentCoreConstants::flowEffectCategory,
+        {},
+        typeName == "None" ? 100 : 140,
+        [typeName](const SelectionContext &context)
+        { ModelNodeOperations::addFlowEffect(context, typeName); },
+    &isFlowTransitionItem));
+}
+
+void DesignerActionManager::addCustomTransitionEffectAction()
+{
+    addDesignerAction(new ModelNodeContextMenuAction(
+        QByteArray(ComponentCoreConstants::flowAssignEffectCommandId),
+        ComponentCoreConstants::flowAssignCustomEffectDisplayName,
+        {},
+        ComponentCoreConstants::flowEffectCategory,
+        {},
+        80,
+        &ModelNodeOperations::addCustomFlowEffect,
+    &isFlowTransitionItem));
+}
+
 DesignerActionToolBar::DesignerActionToolBar(QWidget *parentWidget) : Utils::StyledBar(parentWidget),
     m_toolBar(new QToolBar("ActionToolBar", this))
 {
+    QWidget* empty = new QWidget();
+    empty->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Preferred);
+    m_toolBar->addWidget(empty);
+
     m_toolBar->setContentsMargins(0, 0, 0, 0);
     m_toolBar->setFloatable(true);
     m_toolBar->setMovable(true);
@@ -1063,10 +1372,10 @@ DesignerActionToolBar::DesignerActionToolBar(QWidget *parentWidget) : Utils::Sty
 
     auto horizontalLayout = new QHBoxLayout(this);
 
-    horizontalLayout->setMargin(0);
+    horizontalLayout->setContentsMargins(0, 0, 0, 0);
     horizontalLayout->setSpacing(0);
 
-    horizontalLayout->setMargin(0);
+    horizontalLayout->setContentsMargins(0, 0, 0, 0);
     horizontalLayout->setSpacing(0);
 
     horizontalLayout->addWidget(m_toolBar);
